/**
 * Defines the functionality of the master module in the airline
 * reservation system.
 **/

#ifndef MASTER_MODULE_H
#define MASTER_MODULE_H

#include <systemc>
#include <tlm.h>
#include <map>

#include "bus_types.h"
#include "basic_module.h"
#include "basic_protocol.h"
#include "bounded_queue.h"

#ifdef PROPERTY_2
#include "property2.h"
#endif

using user::bounded_queue;
using namespace basic_protocol;
using tlm::tlm_tag;



namespace airline{

  class master_module : 
    public sc_module,
    public user::basic_module
    
  {


  public: 
    // The constructor
    master_module (sc_module_name module_name, unsigned int q_size);
    
    SC_HAS_PROCESS(master_module);
    
    nb_in_port_t in_port_planner; // Input port from the planner
    nb_in_port_t in_port_io1;     // Input port from the first IO module

    nb_out_port_t out_port_planner; // Outbound port to the flight planner
    nb_out_port_t out_port_io1; // Outbound port to first IO module

    sc_in_clk clk; // The module clock

    static bool obj_eq(const TRANS_TYPE, const TRANS_TYPE);
    static void print_function(const TRANS_TYPE);

    virtual void before_end_of_elaboration();

  protected:

    // The SC_THREAD's 
    void get_new_requests(void);
    void request_planning(void);
    void receive_planning(void);
    void send_planning(void);

    void handle_special_request(TRANS_TYPE, PAYLD_TYPE);

    
    // The queues
    bpq_t new_req;
    bpq_t plan_done;

    bpq_t plan_pend;
    bpq_t reserv_pend;

    // The corresponding events
    sc_event new_req_nonempty;
    sc_event new_req_nonfull;
    sc_event plan_done_nonempty;
    sc_event plan_done_nonfull;

    enum stage_t {
      Q_NEW_REQ,
      PLANNING,
      Q_PLAN_DONE,
      CANCELED,
      RETURNED
    };

    std::map<int, stage_t> stagemap;
    
    unsigned int queue_size;

#ifdef PROPERTY_2
    vect_mon_t* prop2;
#endif

  }; //class

}; //namespace

#endif
