#include "request.h"

using airline::request;

unsigned int request::unique_id = 0;
request* request::undefined_request = new request();


/**
 * Constructor
 */
request::request(int src, int dst, int dat, int dat2, 
		 int sts, int cat, int prt):
  connecting_flights(new list_t()),
  connecting_flights2(new list_t())

{
  source = src;
  destination = dst;
  date = dat;
  date2 = dat2;
  seats = sts;
  category = cat;
  priority = prt;
  is_speculative = false;
  
  
  // Set the internal data 
  request_id = unique_id++;
  birth = sc_delta_count();
  req_sequence = 0; // Initially, no sequence number
  returnflight = date2 > date;
  
#ifdef DEBUG_REQUEST_H
  printf("DEBUG_REQUEST is turned on.\n");
  print_me();
#endif
}


/**
 * Constructor
 */
request::request() {
  request(REQUEST_UNDEF_VAL, // Source
	  REQUEST_UNDEF_VAL, // Destination
	  REQUEST_UNDEF_VAL, // Date
	  REQUEST_UNDEF_VAL, // Date for return flight
	  REQUEST_UNDEF_VAL, // Seats
	  REQUEST_UNDEF_VAL, // Category of service
	  REQUEST_UNDEF_VAL);// Priority

}

/**
 * Constructor with supplied id (for use when sending cancelation
 * requests.
 */
request::request(int id) {
  request();
  request_id = id;
#ifdef DEBUG_REQUEST_H
  printf("DEBUG_REQUEST is turned on.\n");
  printf("request_id changed in the constructor.\n");
  print_me();
#endif
}



/**
 * Returns the calculated priority of the request based on its initial
 * priority and its time since birth.
 */
int request::get_priority() const {
  sc_dt::uint64 current = sc_delta_count();
  sc_dt::uint64 delta = current - birth;
  int reported = delta;
  reported = reported + priority;;

#ifdef DEBUG_REQUEST_H
  print_me();
  cout << "request " << request_id << ", current = " << current << ", delta = " 
       << delta << ", reported = " << reported << endl;
#endif
  return reported;
}


/**
 * Pretty print
 */
void request::print_me() const {
  printf("request %d / seq %d (src=%d, dst=%d, dat=%d, dat2=%d, sts=%d, cat=%d, prt=%d, birth=%d)(%s)\n", 
	 request_id, req_sequence, source, destination, 
	 date, date2, seats, category, priority, (int) birth,
	 is_return() ? "return" : "1-way");

  if (! connecting_flights->empty()) {
    printf("                                           Legs: ");
    std::list< pair_t* >::iterator it;

    for (it = connecting_flights->begin();
	 it != connecting_flights->end();
	 it++) {
      
      cout << "(" << (*it)->first << ", " << (*it)->second << ") ";
    }
    cout << endl;
  }  

  if (! connecting_flights2->empty()) {
    printf("                                           Return trip: ");
    std::list< pair_t* >::iterator it;

    for (it = connecting_flights2->begin();
	 it != connecting_flights2->end();
	 it++) {
      
      cout << "(" << (*it)->first << ", " << (*it)->second << ") ";
    }
    cout << endl;
  }

  cout << "Speculative request? " << (is_speculative? "Yes" : "No") << endl;

}

/**
 * Generates a new list by reversing the connecting flight in the
 * supplied list
 */
list_t* request::reverse_connecting_flights (list_t* c) {
  if (c->empty()) {
    fprintf(stderr, "request::reverse_connecting_flights() cowardly refuses to reverse an empty flight list.\n");
    exit(3);
  }

  std::list< pair_t* >::iterator it;

  list_t* ret = new list_t();
  
  for (it = c->begin();
       it != c->end();
       it++) {

    pair_t* legrev = new pair_t((*it)->second, (*it)->first );
    
    ret->push_front(legrev);
  }
  
  return ret;
}
