/**
 * A class that encapsulates the data related to a request. Initially
 * I was considering making this into a struct rather than a class to
 * make sure that the request is not more clever than it should
 * be. However, a class allows to add sanity checks and other code
 * that will help in the debugging efforts. So, a class it is then. 
 */

#ifndef REQUEST_H
#define REQUEST_H

#include <stdio.h>
#include <iostream>
#include <systemc.h>
#include <list>
#include <algorithm>

#define REQUEST_UNDEF_VAL  -1
#define REQUEST_ERROR_VAL  -10

typedef std::pair< int, int > pair_t;
typedef std::list< pair_t* > list_t;

namespace airline {
  class request {
    
  public: 
    
    
    // Constructors
    request();
    request(int src, int dst, int dat, int dat2, int sts, int cat, int prt);
    request(int id);
    
    void print_me() const;
     
    /**
     * The usual setters
     */
    void set_source(unsigned int src) {source = src;}
    void set_destination(unsigned int dst) {destination = dst;}
    void set_date(unsigned int dat) {date = dat;}
    void set_seats(unsigned int st) {seats = st;}
    void set_category(unsigned int cat) {category = cat;}
    void set_connecting_flights(list_t* conn) {connecting_flights = conn;}
    void set_connecting_flights2(list_t* conn) {connecting_flights2 = conn;}
    void set_req_sequence(unsigned int i) {req_sequence = i;}
    void update_priority(int pri) {priority += pri;}
    void set_speculative_bit_to(bool val) {is_speculative = val;}
    

    /**
     * The usual getters
     */
    unsigned int get_source() const {return source;}
    unsigned int get_destination() const {return destination;}
    unsigned int get_date() const {return date;}
    unsigned int get_date2() const {return date2;}
    unsigned int get_seats() const {return seats;}
    unsigned int get_category() const {return category;}
    unsigned int get_request_id() const {return request_id;}
    unsigned int get_req_sequence() const {return req_sequence;}
    list_t* get_connecting_flights() const {return connecting_flights;};
    list_t* get_connecting_flights2() const {return connecting_flights2;};
    bool is_return() const {return returnflight;}
    bool get_speculative_status() const {return is_speculative;}

    
    /**
     * Some functions for internal use
     */
    static list_t* reverse_connecting_flights(list_t* c);

    

    
    /**
     * This getter returns a changing value based on the simulation
     * clock. This allows us to increase the priority of requests that
     * have been pending for a while.
     */
    int get_priority () const;

    static request* undefined_request;



   
  protected: 

    // Variables related to the trip
    int request_id;  // Unique id of each request
    int source;      // Starting point of the trip
    int destination; // Ending point
    int date;        // Travel date
    int date2;       // Return date
    int seats;       // Number of passengers
    int category;    // Category of service
    bool returnflight; 
    bool is_speculative;
    
    int price;       // Cost of trip
    int ccnumber;    // Credit card number
    int confnumber;  // Confirmation number
   
    // Lists of flights are lists of pairs of integers
    list_t* connecting_flights;
    list_t* connecting_flights2; // Return trip
    
    
    // A corresponding object containing prices

    sc_dt::uint64 birth;       // Set to be the delta cycle at which the request is created
    int priority;              // Priority assigned to this request
    unsigned int req_sequence; // Sequence number for split requests

    sc_dt::uint64 get_birth() const {return birth;}

  private:
    static unsigned int unique_id;

  }; // request

}; // namespace


/**
ostream &operator<<(ostream &fsout, const airline::request req) {
  fsout << "From " << req.get_source() << ", to " << req.get_destination() 
	<< ", on " << req.get_date() << ", " << req.get_seats() 
	<< " seats, cat: " << req.get_category() << endl;
  return fsout;
}
*/


#endif
