/**
 * Defines the datatypes and interfaces for communication between the
 * test generator and the IO modules. Based on basic_protocol.h
 *
 * Initial creation date: January 2008
 */

#ifndef TEST_PROTOCOL_H
#define TEST_PROTOCOL_H

#include <systemc.h>
#include "request.h"
#include "basic_protocol.h"

namespace basic_protocol {

  
  /**
   * The interface for communicating test data. 
   */
  template< typename TOKEN_TYPE, typename RESPONSE_TYPE >

    class test_if: public sc_interface {
    public:
      virtual RESPONSE_TYPE receive_transaction(const TOKEN_TYPE*) = 0;
    }; // interface
  

  /**
   * Test initiator port
   * 
   * The port from which to drive the exports in the slave modules.
   */
  template< typename TOKEN_TYPE, typename RESPONSE_TYPE >
  class test_initiator_port : 
    public sc_port < test_if < TOKEN_TYPE, RESPONSE_TYPE > >
  {
    
  public:
    typedef test_if < TOKEN_TYPE, RESPONSE_TYPE > if_type;
    
    // The constructor
    test_initiator_port (const char* port_name) : 
      sc_port< if_type > (port_name) 
    {}
    
  }; // test_initiator_port
}; //namespace

#endif
