#ifndef BASIC_CHANNEL_H
#define BASIC_CHANNEL_H

#include <systemc.h>
#include "tlm.h"
#include "basic_protocol.h"
#include "basic_module.h"

using tlm::tlm_nonblocking_get_if;
using tlm::tlm_nonblocking_put_if;
using tlm::tlm_fifo;
using tlm::tlm_tag;

using basic_protocol::basic_request;
using user::basic_module;


namespace user {
  
  template< typename TRANS, typename PAYLD >
  class basic_channel : 
    public sc_module,
    virtual public basic_module,
    public tlm_nonblocking_get_if <basic_request<TRANS,PAYLD>* >,
    public tlm_nonblocking_put_if <basic_request<TRANS,PAYLD>* > {

    
  public:

    typedef basic_request<TRANS,PAYLD>* request_t;
    

    tlm_fifo< request_t >* internal_fifo; // The embedded tlm_fifo
    
    // The constructor with explicit channel width
    basic_channel(sc_module_name module_name, unsigned int width = 0) : 
      sc_module(module_name){
      internal_fifo = new tlm_fifo< request_t >(width);
    }

    
    
    // Implement the functions from tlm_non_blocking_get_if
    bool nb_get(request_t &r) {
      bool return_val = internal_fifo->nb_get(r);
      dprint(9, "Basic channel \"%s\": nb_get() will return %d\n", name(), return_val);
      CONDPRINT(9, internal_fifo->debug());
      return return_val;
    }


    bool nb_can_get (tlm_tag< request_t >* t) const {
      bool return_val = internal_fifo->nb_can_get(t);
      dprint(9, "Basic channel %s: nb_can_get() will return %d\n", name(), return_val);
      CONDPRINT(9, internal_fifo->debug());
      return return_val;
    }

    const sc_event &ok_to_get(tlm_tag< request_t >* t) const {
      dprint(9, "Basic channel \"%s\": Somebody is asking for the ok_to_get signal.\n", name());
      return internal_fifo->ok_to_get(t);
    }
    

    // Implement the functions from the tlm_nonblocking_put_if
    bool nb_put (const request_t &t ) {
      dprint(9, "Basic channel \"%s\": running nb_put (id = %d)\n", name(), t->get_request()->get_request_id());
      bool return_val = internal_fifo->nb_put(t);
      dprint(9, "Basic channel \"%s\": nb_put will return %d\n", name(), return_val);
      CONDPRINT(9, internal_fifo->debug());
      return return_val;
    }

    bool nb_can_put (tlm_tag< request_t >* t) const {
      bool return_val = internal_fifo->nb_can_put(t);
      dprint(9, "Basic channel \"%s\": nb_can_put() will return %d\n", name(), return_val);
      return return_val;
    }

    const sc_event &ok_to_put(tlm_tag< request_t > *t) const {
      dprint(9, "Basic channel \"%s\": Somebody is asking for the the ok_to_put signal.\n", name());
      return internal_fifo->ok_to_put(t);
    }
    
  }; //basic_channel
    
    
  

}; // namespace


#endif
