/**
 * Defines the functionality of the input modules for the system
 **/

#ifndef TEST_GENERATOR_H
#define TEST_GENERATOR_H

#include "basic_module.h"
#include "bus_types.h"
#include "test_protocol.h"
#include "request.h"
#include "basic_protocol.h"

using basic_protocol::status_t;

namespace user{

  class test_generator : 
    public sc_module,
    public basic_module, 
    public virtual basic_protocol::test_if < token_t, status_t >
  {
    
  public: 


    // The constructor
    test_generator (sc_module_name module_name );
    
    SC_HAS_PROCESS(test_generator);
    
    // The IO modules will contact us through the exports declared here
    sc_export < basic_protocol::test_if< token_t, status_t > > io1_in_port;

    // We will contact the IO modules through the initiator ports
    // declared here
    basic_protocol::test_initiator_port< token_t, status_t > io1_out_port;
    
    // Concrete implementation of the communication function defined
    // in test_if
    status_t receive_transaction(const token_t*);
    
    // Module clock
    sc_in_clk clk;

  protected:
    
    // Sends test vectors to the IO modules
    void send_to_io(); 

  }; //class

}; //namespace

#endif
