#ifdef PROPERTY_2

#include "property2.h"


unsigned int property2::unique_id = 0;


/**
 * The constructor
 */
property2::property2(mon_observer* obs) : mon_prototype() {
  observer = obs;

  max_allowed_delay = sc_time(250, SC_NS);
  max_allowed_deadtime = sc_time(250, SC_NS);
  my_id = unique_id++;

#ifdef PROPERTY_2_VERBOSE
  printf("Registering a callback\n");
#endif
  obs->register_monitor(this, MON_TIMED_NOTIFY_PHASE_END);

#ifdef PROPERTY_2_VERBOSE
  printf("Property 2 (instance %d) initialized\n", my_id);
#endif
} // Constructor


const char*
property2:: to_string() const {
  return "Liveness property for requests.";
}

/** 
 * Called by the io_module as soon as it receives a new transaction
 */
void 
property2::io_module_receive_transaction(int transaction_id) {
#ifdef PROPERTY_2_VERBOSE
  printf("property2::%s(%d) starts\n", __FUNCTION__, transaction_id);
#endif

  // At this point the transaction should not have an association
  assert (requesttime.find(transaction_id) == requesttime.end());

#ifdef PROPERTY_2_VERBOSE
  print_request_summary();
#endif

  // Store the time when the transaction entered the io_module
  requesttime[ transaction_id ]= sc_time_stamp();
#ifdef PROPERTY_2_VERBOSE
  requestmodule [ transaction_id ] = "io_module";
  printf("property2::%s(%d) finished\n", __FUNCTION__, transaction_id);
#endif
}




/** 
 * Called by the io_module as soon as it pushes a transaction to the
 * master module through the channel
 */
void 
property2::io_module_send_to_master(int transaction_id) {
#ifdef PROPERTY_2_VERBOSE
  printf("property2::%s(%d) starts\n", __FUNCTION__, transaction_id);
#endif
  
  // At this point the transaction should have an association
  assert (requesttime.find(transaction_id) != requesttime.end());
  
  sc_time prev_time = requesttime[ transaction_id ];
  sc_time curr_time = sc_time_stamp();

#ifdef PROPERTY_2_VERBOSE
  print_request_summary();
#endif

  assert( (curr_time - prev_time ) <= max_allowed_delay);

  // Store the time when the transaction entered the io_module
  requesttime[ transaction_id ]= curr_time;

#ifdef PROPERTY_2_VERBOSE
  requestmodule [ transaction_id ] = "io_module -> master_module";
  printf("property2::%s(%d) finished\n", __FUNCTION__, transaction_id);
#endif
}


void 
property2::io_module_receive_from_master(int transaction_id) {
#ifdef PROPERTY_2_VERBOSE
  printf("property2::%s(%d) starts\n", __FUNCTION__, transaction_id);
#endif  
  // At this point the transaction should have an association
  assert (requesttime.find(transaction_id) != requesttime.end());
  
  sc_time prev_time = requesttime[ transaction_id ];
  sc_time curr_time = sc_time_stamp();

#ifdef PROPERTY_2_VERBOSE
  print_request_summary();
#endif
  assert( (curr_time - prev_time ) <= max_allowed_delay);

  // This request is finished. Remove from queue
  requesttime.erase(transaction_id);

#ifdef PROPERTY_2_VERBOSE
  requestmodule.erase(transaction_id);
  printf("property2::%s(%d) finished\n", __FUNCTION__, transaction_id);
#endif
}


void 
property2::master_receive_request_from_io(int transaction_id) {
#ifdef PROPERTY_2_VERBOSE
  printf("property2::%s(%d) starts\n", __FUNCTION__, transaction_id);
#endif
  
  // At this point the transaction should have an association
  assert (requesttime.find(transaction_id) != requesttime.end());
  
  sc_time prev_time = requesttime[ transaction_id ];
  sc_time curr_time = sc_time_stamp();
#ifdef PROPERTY_2_VERBOSE
  print_request_summary();
#endif

  assert( (curr_time - prev_time ) <= max_allowed_delay);

  // Store the time when the transaction entered the io_module
  requesttime[ transaction_id ]= curr_time;

#ifdef PROPERTY_2_VERBOSE
  requestmodule [ transaction_id ] = "master_module (from io)";
  printf("property2::%s(%d) finished\n", __FUNCTION__, transaction_id);
#endif
}



void 
property2::master_send_to_flight_planner(int transaction_id) {
#ifdef PROPERTY_2_VERBOSE
  printf("property2::%s(%d) starts\n", __FUNCTION__, transaction_id);
#endif

  // At this point the transaction should have an association
  assert (requesttime.find(transaction_id) != requesttime.end());
  
  sc_time prev_time = requesttime[ transaction_id ];
  sc_time curr_time = sc_time_stamp();
#ifdef PROPERTY_2_VERBOSE
  print_request_summary();
#endif
  assert( (curr_time - prev_time ) <= max_allowed_delay);

  // Store the time when the transaction entered the io_module
  requesttime[ transaction_id ]= curr_time;

#ifdef PROPERTY_2_VERBOSE 
  requestmodule [ transaction_id ] = "master_module -> flight_planner";
  printf("property2::%s(%d) finished\n", __FUNCTION__, transaction_id);
#endif
}


void 
property2::flight_planner_receive_request_from_master(int transaction_id) {
#ifdef PROPERTY_2_VERBOSE
  printf("property2::%s(%d) starts\n", __FUNCTION__, transaction_id);
#endif
  
  // At this point the transaction should have an association
  assert (requesttime.find(transaction_id) != requesttime.end());
  
  sc_time prev_time = requesttime[ transaction_id ];
  sc_time curr_time = sc_time_stamp();

#ifdef PROPERTY_2_VERBOSE  
  print_request_summary();
#endif
  assert( (curr_time - prev_time ) <= max_allowed_delay);

  // Store the time when the transaction entered the io_module
  requesttime[ transaction_id ]= curr_time;


#ifdef PROPERTY_2_VERBOSE
  requestmodule [ transaction_id ] = "flight_planner (from master)";  
  printf("property2::%s(%d) finished\n", __FUNCTION__, transaction_id);
#endif
}



void 
property2::flight_planner_send_request_to_master(int transaction_id) {
#ifdef PROPERTY_2_VERBOSE
  printf("property2::%s(%d) starts\n", __FUNCTION__, transaction_id);
#endif  
  // At this point the transaction should have an association
  assert (requesttime.find(transaction_id) != requesttime.end());
  
  sc_time prev_time = requesttime[ transaction_id ];
  sc_time curr_time = sc_time_stamp();

#ifdef PROPERTY_2_VERBOSE  
  print_request_summary();
#endif
  assert( (curr_time - prev_time ) <= max_allowed_delay);

  // Store the time when the transaction entered the io_module
  requesttime[ transaction_id ]= curr_time;

#ifdef PROPERTY_2_VERBOSE
  requestmodule [ transaction_id ] = "flight_planner -> master_module";
  printf("property2::%s(%d) finished\n", __FUNCTION__, transaction_id);
#endif
}


void 
property2::master_receive_request_from_flight_planner(int transaction_id) {
#ifdef PROPERTY_2_VERBOSE
  printf("property2::%s(%d) starts\n", __FUNCTION__, transaction_id);
#endif
  
  // At this point the transaction should have an association
  assert (requesttime.find(transaction_id) != requesttime.end());
  
  sc_time prev_time = requesttime[ transaction_id ];
  sc_time curr_time = sc_time_stamp();

#ifdef PROPERTY_2_VERBOSE  
  print_request_summary();
#endif

  assert( (curr_time - prev_time ) <= max_allowed_delay);

  // Store the time when the transaction entered the io_module
  requesttime[ transaction_id ]= curr_time;

#ifdef PROPERTY_2_VERBOSE
  requestmodule [ transaction_id ] = "master (plan done)";
  printf("property2::%s(%d) finished\n", __FUNCTION__, transaction_id);
#endif
}



void 
property2::master_send_request_to_io_module(int transaction_id) {
#ifdef PROPERTY_2_VERBOSE
  printf("property2::%s(%d) starts\n", __FUNCTION__, transaction_id);
#endif  
  // At this point the transaction should have an association
  assert (requesttime.find(transaction_id) != requesttime.end());
  
  sc_time prev_time = requesttime[ transaction_id ];
  sc_time curr_time = sc_time_stamp();

#ifdef PROPERTY_2_VERBOSE  
  print_request_summary();
#endif

  assert( (curr_time - prev_time ) <= max_allowed_delay);

  // Store the time when the transaction entered the io_module
  requesttime[ transaction_id ]= curr_time;

#ifdef PROPERTY_2_VERBOSE  
  requestmodule [ transaction_id ] = "master_module -> io_module";
  printf("property2::%s(%d) finished\n", __FUNCTION__, transaction_id);
#endif
}


/**
 * Prints a summary for all requests in the database
 */
void 
property2::print_request_summary() {
  printf("property2::%s() starts\n", __FUNCTION__);

#ifdef PROPERTY_2_VERBOSE
  printf ("Reporting for monitor #%d\n", my_id);

  assert(requestmodule.size() == requesttime.size());

  misc_time::const_iterator it = requesttime.begin();
  mistring::const_iterator is = requestmodule.begin();

  std::cout << "The time is " << sc_time_stamp() << std::endl;
  printf("\trequest\ttime\twhere\n");
  while (it != requesttime.end()) {
    assert (it->first == is->first);
    std::cout << '\t' << it->first << '\t' << it->second << '\t' << is->second << std::endl;
    it++;
    is++;
  }
#endif
  printf("property2::%s() finished\n", __FUNCTION__);
}



/**
 * No request is blocked inside a channel/module for more than some
 * preset period of time
 */
void 
property2::callback_timed_notify_phase_end() {
#ifdef PROPERTY_2_VERBOSE  
  printf("property2::%s() starts\n", __FUNCTION__);
#endif

  misc_time::const_iterator it = requesttime.begin();
  sc_time now = sc_time_stamp();

#ifdef PROPERTY_2_VERBOSE
  print_request_summary();
  std::cout << "The time is " << now << std::endl;
#endif
  
  while (it != requesttime.end()) {
    sc_time this_req_time = it->second;
    assert (now - this_req_time < max_allowed_deadtime);
    it++;
  }
#ifdef PROPERTY_2_VERBOSE  
  printf("property2::%s() finished\n", __FUNCTION__);
#endif  
}


#endif
